package ludumdare19.gameobjects
{
    import flash.display.Bitmap;
    import flash.display.PixelSnapping;
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import ludumdare19.Sfx;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.gameobject.PlatformerCharacter;
    import stego.util.collision.CollisionInfo;
    import stego.util.collision.HitBox;
    import stego.util.graphics.AnimatedSpriteSheet;
    import stego.util.ObjectPool;
    import stego.util.StringUtil;

    public class Player extends PlatformerCharacter
    {
        [Embed(source = '../../../assets/arrow.png')]
        protected static const ARROW_GFX_CLASS:Class;

        [Embed(source='../../../assets/player.png')]
        protected static const _PLAYER_SPRITESHEET_CLASS:Class;
        protected static const PLAYER_SPRITESHEET_DATA:Object = { bitmapData:new _PLAYER_SPRITESHEET_CLASS().bitmapData,
                                                                  frameWidth:50,
                                                                  frameHeight:50 };
        protected static const COLLISION_SIZE:Number = 10;
        protected static const CLICK_SHOOT_SPEED:Number = 550;
        protected static const DRAG_DOWN_TO_SPEED:Number = 260;
        protected static const DRAG_ACCEL:Number = (CLICK_SHOOT_SPEED-DRAG_DOWN_TO_SPEED) / 0.5;
        protected static const MOUSE_SEEK_ACCEL:Number = 150;
        protected static const BOUNCE:Number = 0.75;
        protected static const BOUNCE_FRICTION:Number = 0.95;

        public static const WHITE:String   = "white";
        public static const RED:String     = "red";
        public static const GREEN:String   = "green";
        public static const BLUE:String    = "blue";
        public static const RAINBOW:String = "rainbow";

        public function get color():String { return _color; }
        public function set color(value:String):void { _color = value; updateAnim(); }
        protected var _color:String = WHITE;

        public function get charged():Boolean { return _charged; }
        public function set charged(value:Boolean):void { _charged = value; updateAnim(); }
        protected var _charged:Boolean = true;

        protected var arrow:Sprite;

        /** util for rendering sprites out of a spritesheet */
        protected var _spriteSheet:AnimatedSpriteSheet;

        public override function init(game:GameInterface):void
        {
            super.init(game);

            createGraphics();
            restart();
            stage.addEventListener(MouseEvent.MOUSE_DOWN, mouseDown);
        }

        /**
         * set initial values after spawning
         */
        override public function restart():void
        {
            super.restart();
            _color = WHITE;
            _charged = true;
            updateAnim();
        }

        /**
         * The user clicked the stage!  Shoot the player around!
         */
        private function mouseDown(e:MouseEvent):void
        {
            if (_charged || color == Player.RAINBOW)
            {
                charged = false || color == Player.RAINBOW;

                var mouseDist:Number = Math.sqrt( (mouseX * mouseX) + (mouseY * mouseY) );
                if (mouseDist < 1)
                {
                    dx = CLICK_SHOOT_SPEED;
                    dy = 0;
                }
                else
                {
                    dx = mouseX * CLICK_SHOOT_SPEED / mouseDist;
                    dy = mouseY * CLICK_SHOOT_SPEED / mouseDist;
                }
                Sfx.boost();
            }
            else
            {
                Sfx.boostFail();
            }
        }

        protected function createGraphics():void
        {
            if (!_spriteSheet)
            {
                var bitmap:Bitmap = new Bitmap(null, PixelSnapping.NEVER);
                addChild(bitmap);
                _spriteSheet = new AnimatedSpriteSheet( bitmap,
                                                        PLAYER_SPRITESHEET_DATA.bitmapData,
                                                        PLAYER_SPRITESHEET_DATA.frameWidth,
                                                        PLAYER_SPRITESHEET_DATA.frameHeight );
                _spriteSheet.addAnimation("white",       [2,3], 32.0, true);
                _spriteSheet.addAnimation("whitebright", [0,1], 32.0, true);
                _spriteSheet.addAnimation("red",         [6,7], 32.0, true);
                _spriteSheet.addAnimation("redbright",   [4,5], 32.0, true);
                _spriteSheet.addAnimation("green",       [10,11], 32.0, true);
                _spriteSheet.addAnimation("greenbright", [8,9], 32.0, true);
                _spriteSheet.addAnimation("blue",        [14,15], 32.0, true);
                _spriteSheet.addAnimation("bluebright",  [12,13], 32.0, true);
                _spriteSheet.addAnimation("rainbow",       [16,17,18,19,20,21,22,23,24,25,26,27], 60.0, true);
                _spriteSheet.addAnimation("rainbowbright", [16,17,18,19,20,21,22,23,24,25,26,27], 60.0, true);
                bitmap.x = -0.5 * bitmap.width;
                bitmap.y = -0.5 * bitmap.height;
            }

            if (!arrow)
            {
                arrow = new Sprite();
                addChild(arrow);
                var arrowGfx:Bitmap = new ARROW_GFX_CLASS();
                arrowGfx.x = 25;
                arrowGfx.y = -0.5 * arrowGfx.height;
                arrow.addChild(arrowGfx);
            }
        }

        /**
         * set the animation in the spritesheet to match the new color/chargedness
         */
        protected function updateAnim():void
        {
            var animname:String = _color;
            if (_charged)
            {
                animname += "bright";
            }
            _spriteSheet.playAnimation(animname);
        }

        /**
         * cleanup for when we are done with this instance for a while.
         * May still get reused!
         */
        override public function shutdown():void
        {
            stage.removeEventListener(MouseEvent.MOUSE_DOWN, mouseDown);
            super.shutdown();
        }

        /**
         * get this gameobject's hitboxes.
         * The Array returned by this can be returned with stego.util.ObjectPool.returnArray(a, stego.util.collision.HitBox)
         * Uses getBounds by default! Override if you do your own hitboxes!
         */
        override public function getHitboxes():Array
        {
            var a:Array = ObjectPool.getArray(HitBox, 1);
            var hb:HitBox = a[0];
            hb.userData = "";
            hb.color = 0x00FF00;
            hb.x = -0.5 * COLLISION_SIZE;
            hb.y = -0.5 * COLLISION_SIZE;
            hb.width = COLLISION_SIZE;
            hb.height = COLLISION_SIZE;
            return a;
        }

        /**
         * per-frame updates
         * @param	elapsedTime - how long the last frame was
         */
        override public function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);
            _spriteSheet.update(elapsedTime);

            var mouseDir:Number = Math.atan2(mouseY, mouseX);
            arrow.rotation = mouseDir * 180.0 / Math.PI;

            ddx += MOUSE_SEEK_ACCEL * Math.cos(mouseDir);
            ddy += MOUSE_SEEK_ACCEL *  Math.sin(mouseDir);

            var speed:Number = Math.sqrt(dx*dx + dy*dy);
            if (speed > DRAG_DOWN_TO_SPEED)
            {
                ddx -= DRAG_ACCEL * dx / speed;
                ddy -= DRAG_ACCEL * dy / speed;
            }
        }

        /**
         * overriding collision respons to do bouncing
         */
        override public function collisionOccurred(ci:CollisionInfo, object:GameObject):void
        {
            if (!canPassThrough(ci, object))
            {
                var collisionBox:Rectangle = ci.hitbox1;

                var collision_dx:Number = (null != object) ? object.dx : 0;
                var collision_dy:Number = (null != object) ? object.dy : 0;

                if ( ci.normal.x > 0.0 )
                {
                    x = ci.location.x - collisionBox.left;
                    if (dx < collision_dx) { dx = (BOUNCE + 1.0) * collision_dx - BOUNCE * dx; }
                    if (ddx < 0) { ddx = 0; }
                    dy *= BOUNCE_FRICTION;
                }
                else if ( ci.normal.x < 0.0 )
                {
                    x = ci.location.x - collisionBox.right;
                    if (dx > collision_dx) { dx = (BOUNCE + 1.0) * collision_dx - BOUNCE * dx; }
                    if (ddx > 0) { ddx = 0; }
                    dy *= BOUNCE_FRICTION;
                }

                if ( ci.normal.y > 0.0  )
                {
                    y = ci.location.y - collisionBox.top;
                    if (dy < collision_dy) { dy = (BOUNCE + 1.0) * collision_dy - BOUNCE * dy; }
                    if (ddy < 0) { ddy = 0; }
                    dx *= BOUNCE_FRICTION;
                }
                else if ( ci.normal.y < 0.0 )
                {
                    y = ci.location.y - collisionBox.bottom;
                    if (dy > collision_dy) { dy = (BOUNCE + 1.0) * collision_dy - BOUNCE * dy; }
                    if (ddy > 0) { ddy = 0; }
                    dx *= BOUNCE_FRICTION;
                }

                isOnGround = isOnGround || (ci.normal.y < -Math.SQRT1_2);


                // color changing
                var colorChanged:Boolean = false;
                if (color != Player.RAINBOW)
                {
                    switch (ci.hitbox2.userData)
                    {
                        case "redorb":
                            if (color != Player.RED)
                            {
                                color = Player.RED;
                                colorChanged = true;
                            }
                            break;
                        case "greenorb":
                            if (color != Player.GREEN)
                            {
                                color = Player.GREEN;
                                colorChanged = true;
                            }
                            break;
                        case "blueorb":
                            if (color != Player.BLUE)
                            {
                                color = Player.BLUE;
                                colorChanged = true;
                            }
                            break;
                    }
                }

                if (colorChanged)
                {
                    Sfx.changeColor();
                }
                else if (!_charged)
                {
                    Sfx.getCharged();
                }
                else
                {
                    if ( (Math.abs(ci.normal.x) > Math.abs(ci.normal.y)) && Math.abs(dx) > 100.0)
                    {
                        Sfx.bounce();
                    }
                    else if ( (Math.abs(ci.normal.x) <= Math.abs(ci.normal.y)) && Math.abs(dy) > 100.0)
                    {
                        Sfx.bounce();
                    }
                }

                // recharge on bounces
                charged = true;
            }
        }

        /**
         * override to let yourself go through things
         */
        override public function canPassThrough(ci:CollisionInfo, object:GameObject):Boolean
        {
            if ( (null == ci.hitbox2.userData) || ("" == ci.hitbox2.userData))
            {
                return false;
            }
            if (StringUtil.endsWith(ci.hitbox2.userData, "ghost"))
            {
                return true;
            }
            if ("towlr-bg" == ci.hitbox2.userData)
            {
                return true;
            }

            return false;
        }
    }
}
