package ludumdare19.gameobjects
{
    import stego.GameObject;
    import stego.gameobject.TileMap;
    import stego.util.collision.CollisionInfo;
    import stego.util.collision.CollisionUtil;
    import stego.util.collision.HitBox;
    import stego.util.ObjectPool;
	
	/**
     * ...
     * @author Michael Meyer
     */
    public class TileMapWithUserData extends TileMap
    {
        public function setTileWithUserdata(column:uint, row:uint, index:int, userdata:String):void
        {
            super.setTile(column, row, index);
            getTileData(column, row, false).userdata = userdata;
        }
        public function getUserdataAt(row:uint, col:uint):String
        {
            var tileData:Object = getTileData(col, row, false);
            if (tileData == null) {return null;}
            return tileData.userdata;
        }
        override public function clearTile(column:uint, row:uint):void
        {
            var tileData:Object = getTileData(column, row, false);
            if (tileData)
            {
                tileData.userdata = null;
                setTileInternal(tileData, -1);
            }
        }

        /**
         * Exposing the raw tile data for Ludum Dare hacking. BE CAREFUL
         * there's a better way but I'm too tired to think of it.
         */
        public function getAllTileData():Array
        {
            return _tileData;
        }

        /**
         * do a collision check for one hitbox/tile pair
         */
        override protected function collideTile( tileData:Object,
                                        go:GameObject,
                                        hitbox:HitBox,
                                        elapsedTime:Number,
                                        callback:Function = null,
                                        horizontalSnap:Number = 0,
                                        verticalSnap:Number = 0 ):void
        {
            var offsetLeft:Number = (hitbox.x - x - horizontalSnap), // distance from the left edge of the tilemap to the left edge of the object
                offsetTop:Number = (hitbox.y - y - verticalSnap), // distance from the top of the tilemap to the top of the object
                offsetRight:Number = offsetLeft + hitbox.width + (2*horizontalSnap), // distance from the left edge of the tilemap to the right edge of the object
                offsetBottom:Number = offsetTop + hitbox.height + (2 * verticalSnap), // distance from the top of the tilemap to the bottom of the object
                ci:CollisionInfo;
            var tileHb:HitBox = ObjectPool.getObject(HitBox);
            tileHb.userData = tileData.userdata;
            tileHb.x = (tileData.col * _tileWidth);
            tileHb.y = (tileData.row * _tileHeight);
            tileHb.width = _tileWidth;
            tileHb.height = _tileHeight;

            ci = CollisionUtil.checkHitboxCollision(hitbox, go, tileHb, this, elapsedTime, horizontalSnap, verticalSnap);
            if (ci)
            {
                var temp:HitBox;

                // let the character know he hit something
                go.collisionOccurred(ci, this);
                ci.normal.x *= -1;
                ci.normal.y *= -1;
                temp = ci.hitbox1;
                ci.hitbox1 = ci.hitbox2;
                ci.hitbox2 = temp;
                this.collisionOccurred(ci, go);
                ci.normal.x *= -1;
                ci.normal.y *= -1;
                temp = ci.hitbox1;
                ci.hitbox1 = ci.hitbox2;
                ci.hitbox2 = temp;

                // call the callack, if we were given one
                if (null != callback)
                {
                    callback(tileData, this, go, ci);
                }
            }
            ObjectPool.returnObject(HitBox, tileHb);
        }
    }

}