package ludumdare19.rooms
{
    import flash.display.BitmapData;
    import flash.geom.Point;
    import flash.utils.ByteArray;

    /**
     * room data used by RoomLoader
     * @author Mike Meyer
     */
    public class RoomData
    {
        /** room name */
        public var name:String;
        /** xml data for the room */
        public var xml:XML;
        /** static tileset data for the room */
        public var tileSet:BitmapData;

        /** construct a new RoomData */
        public function RoomData( xml:XML,
                                  tileSet:BitmapData = null )
        {
            this.xml = xml;
            this.tileSet = tileSet ? tileSet : TEST_TILEMAP;
        }

        /* helper functions for initialization */

        /** convert a byte array to an XML object */
        private static function byteArrayToXml(ba:ByteArray):XML { return new XML(ba.readUTFBytes( ba.length )); }
        /** helper for init'ing RoomDatas */
        private static function toRoomData(embeddedLevel:Class, tileSet:BitmapData = null):RoomData { return new RoomData(byteArrayToXml(new embeddedLevel()), tileSet); }

        /* tilesets */

        [Embed(source='../../../assets/tileset.png')]
        private static const TEST_TILEMAP_CLASS:Class;
        private static const TEST_TILEMAP:BitmapData = new TEST_TILEMAP_CLASS().bitmapData;

        /* RoomData instances */

        [Embed(source='../../../assets/finalgate.tmx', mimeType='application/octet-stream')]
        private static const _FINAL_GATE:Class;
        public static const FINAL_GATE:RoomData = toRoomData(_FINAL_GATE);

        [Embed(source='../../../assets/start.tmx', mimeType='application/octet-stream')]
        private static const _START:Class;
        public static const START:RoomData = toRoomData(_START);

        [Embed(source='../../../assets/bluemazepart1.tmx', mimeType='application/octet-stream')]
        private static const _BLUEMAZE1:Class;
        public static const BLUEMAZE1:RoomData = toRoomData(_BLUEMAZE1);

        [Embed(source='../../../assets/bluemazepart2.tmx', mimeType='application/octet-stream')]
        private static const _BLUEMAZE2:Class;
        public static const BLUEMAZE2:RoomData = toRoomData(_BLUEMAZE2);

        [Embed(source='../../../assets/greenmaze.tmx', mimeType='application/octet-stream')]
        private static const _GREENMAZE:Class;
        public static const GREENMAZE:RoomData = toRoomData(_GREENMAZE);

        [Embed(source='../../../assets/greenmazelow.tmx', mimeType='application/octet-stream')]
        private static const _GREENMAZELOW:Class;
        public static const GREENMAZELOW:RoomData = toRoomData(_GREENMAZELOW);

        [Embed(source='../../../assets/directed1.tmx', mimeType='application/octet-stream')]
        private static const _DIRECTED1:Class;
        public static const DIRECTED1:RoomData = toRoomData(_DIRECTED1);

        [Embed(source='../../../assets/directed2.tmx', mimeType='application/octet-stream')]
        private static const _DIRECTED2:Class;
        public static const DIRECTED2:RoomData = toRoomData(_DIRECTED2);

        [Embed(source='../../../assets/up+down.tmx', mimeType='application/octet-stream')]
        private static const _UPDOWN:Class;
        public static const UPDOWN:RoomData = toRoomData(_UPDOWN);

        [Embed(source='../../../assets/treasure.tmx', mimeType='application/octet-stream')]
        private static const _TREASURE:Class;
        public static const TREASURE:RoomData = toRoomData(_TREASURE);

        [Embed(source='../../../assets/falling.tmx', mimeType='application/octet-stream')]
        private static const _FALLING:Class;
        public static const FALLING:RoomData = toRoomData(_FALLING);

        [Embed(source='../../../assets/floaters.tmx', mimeType='application/octet-stream')]
        private static const _FLOATERS:Class;
        public static const FLOATERS:RoomData = toRoomData(_FLOATERS);

        [Embed(source='../../../assets/floaters2.tmx', mimeType='application/octet-stream')]
        private static const _FLOATERS2:Class;
        public static const FLOATERS2:RoomData = toRoomData(_FLOATERS2);

        [Embed(source='../../../assets/veni.tmx', mimeType='application/octet-stream')]
        private static const _VENI:Class;
        public static const VENI:RoomData = toRoomData(_VENI);

        [Embed(source='../../../assets/vidi.tmx', mimeType='application/octet-stream')]
        private static const _VIDI:Class;
        public static const VIDI:RoomData = toRoomData(_VIDI);

        [Embed(source='../../../assets/vici.tmx', mimeType='application/octet-stream')]
        private static const _VICI:Class;
        public static const VICI:RoomData = toRoomData(_VICI);

        [Embed(source='../../../assets/notsosecretpassage.tmx', mimeType='application/octet-stream')]
        private static const _NOTSOSECRET:Class;
        public static const NOTSOSECRET:RoomData = toRoomData(_NOTSOSECRET);

        [Embed(source='../../../assets/notsosecretpassagekey.tmx', mimeType='application/octet-stream')]
        private static const _NOTSOSECRETKEY:Class;
        public static const NOTSOSECRETKEY:RoomData = toRoomData(_NOTSOSECRETKEY);

        [Embed(source='../../../assets/thetower0.tmx', mimeType='application/octet-stream')]
        private static const _TOWER0:Class;
        public static const TOWER0:RoomData = toRoomData(_TOWER0);

        [Embed(source='../../../assets/thetower1.tmx', mimeType='application/octet-stream')]
        private static const _TOWER1:Class;
        public static const TOWER1:RoomData = toRoomData(_TOWER1);

        [Embed(source='../../../assets/thetower2.tmx', mimeType='application/octet-stream')]
        private static const _TOWER2:Class;
        public static const TOWER2:RoomData = toRoomData(_TOWER2);

        [Embed(source='../../../assets/thetower3.tmx', mimeType='application/octet-stream')]
        private static const _TOWER3:Class;
        public static const TOWER3:RoomData = toRoomData(_TOWER3);

        [Embed(source='../../../assets/thetower4.tmx', mimeType='application/octet-stream')]
        private static const _TOWER4:Class;
        public static const TOWER4:RoomData = toRoomData(_TOWER4);

        [Embed(source='../../../assets/thetower5.tmx', mimeType='application/octet-stream')]
        private static const _TOWER5:Class;
        public static const TOWER5:RoomData = toRoomData(_TOWER5);

        [Embed(source='../../../assets/thetower6.tmx', mimeType='application/octet-stream')]
        private static const _TOWER6:Class;
        public static const TOWER6:RoomData = toRoomData(_TOWER6);

        [Embed(source='../../../assets/thetower7.tmx', mimeType='application/octet-stream')]
        private static const _TOWER7:Class;
        public static const TOWER7:RoomData = toRoomData(_TOWER7);

        [Embed(source='../../../assets/thetower8.tmx', mimeType='application/octet-stream')]
        private static const _TOWER8:Class;
        public static const TOWER8:RoomData = toRoomData(_TOWER8);

        [Embed(source='../../../assets/thetower9.tmx', mimeType='application/octet-stream')]
        private static const _TOWER9:Class;
        public static const TOWER9:RoomData = toRoomData(_TOWER9);

        [Embed(source='../../../assets/thetower10.tmx', mimeType='application/octet-stream')]
        private static const _TOWER10:Class;
        public static const TOWER10:RoomData = toRoomData(_TOWER10);

        [Embed(source='../../../assets/rightfromtower.tmx', mimeType='application/octet-stream')]
        private static const _RIGHTFROMTOWER:Class;
        public static const RIGHTFROMTOWER:RoomData = toRoomData(_RIGHTFROMTOWER);

        [Embed(source='../../../assets/tennis0.tmx', mimeType='application/octet-stream')]
        private static const _TENNIS0:Class;
        public static const TENNIS0:RoomData = toRoomData(_TENNIS0);

        [Embed(source='../../../assets/tennis1.tmx', mimeType='application/octet-stream')]
        private static const _TENNIS1:Class;
        public static const TENNIS1:RoomData = toRoomData(_TENNIS1);

        [Embed(source='../../../assets/tennis2.tmx', mimeType='application/octet-stream')]
        private static const _TENNIS2:Class;
        public static const TENNIS2:RoomData = toRoomData(_TENNIS2);

        [Embed(source='../../../assets/tennis3.tmx', mimeType='application/octet-stream')]
        private static const _TENNIS3:Class;
        public static const TENNIS3:RoomData = toRoomData(_TENNIS3);

        [Embed(source='../../../assets/falltotennis0.tmx', mimeType='application/octet-stream')]
        private static const _FALLTOTENNIS0:Class;
        public static const FALLTOTENNIS0:RoomData = toRoomData(_FALLTOTENNIS0);

        [Embed(source='../../../assets/falltotennis1.tmx', mimeType='application/octet-stream')]
        private static const _FALLTOTENNIS1:Class;
        public static const FALLTOTENNIS1:RoomData = toRoomData(_FALLTOTENNIS1);

        [Embed(source='../../../assets/secret.tmx', mimeType='application/octet-stream')]
        private static const _SECRET:Class;
        public static const SECRET:RoomData = toRoomData(_SECRET);

        [Embed(source='../../../assets/gate.tmx', mimeType='application/octet-stream')]
        private static const _GATE:Class;
        public static const GATE:RoomData = toRoomData(_GATE);

        [Embed(source='../../../assets/runleft0.tmx', mimeType='application/octet-stream')]
        private static const _RUNLEFT0:Class;
        public static const RUNLEFT0:RoomData = toRoomData(_RUNLEFT0);

        [Embed(source='../../../assets/runleft1.tmx', mimeType='application/octet-stream')]
        private static const _RUNLEFT1:Class;
        public static const RUNLEFT1:RoomData = toRoomData(_RUNLEFT1);

        [Embed(source='../../../assets/runleft2.tmx', mimeType='application/octet-stream')]
        private static const _RUNLEFT2:Class;
        public static const RUNLEFT2:RoomData = toRoomData(_RUNLEFT2);

        [Embed(source='../../../assets/towlr.tmx', mimeType='application/octet-stream')]
        private static const _TOWLR:Class;
        public static const TOWLR:RoomData = toRoomData(_TOWLR);

        [Embed(source='../../../assets/cake.tmx', mimeType='application/octet-stream')]
        private static const _CAKE:Class;
        public static const CAKE:RoomData = toRoomData(_CAKE);

        [Embed(source='../../../assets/realstart.tmx', mimeType='application/octet-stream')]
        private static const _REALSTART:Class;
        public static const REALSTART:RoomData = toRoomData(_REALSTART);

        [Embed(source='../../../assets/realrealstart.tmx', mimeType='application/octet-stream')]
        private static const _REAL2START:Class;
        public static const REAL2START:RoomData = toRoomData(_REAL2START);

        /** all rooms arranged in the grid*/
        public static const ROOMS:Array = [ [null,      null,      null,      null,       null,          null,      null,           TOWER10],
                                            [null,      null,      null,      null,       null,          null,      null,           TOWER9],
                                            [null,      null,      null,      null,       null,          null,      null,           TOWER8],
                                            [null,      null,      null,      null,       null,          null,      TOWER6,         TOWER7, RIGHTFROMTOWER],
                                            [null,      null,      null,      null,       null,          TREASURE,  TOWER5,         TOWER4, TOWER3],
                                            [RUNLEFT2,  RUNLEFT1,  RUNLEFT0,  null,       null,          UPDOWN,    TOWER0,         TOWER1, TOWER2],
                                            [REAL2START,REALSTART, GATE,      START,      GREENMAZE,     DIRECTED1, DIRECTED2],
                                            [SECRET,    BLUEMAZE2, BLUEMAZE1, FINAL_GATE, GREENMAZELOW,  FALLING,   null,           VIDI],
                                            [null,      null,      null,      TOWLR,      FALLTOTENNIS0, FLOATERS,  FLOATERS2,      VENI,   VICI,   NOTSOSECRET],
                                            [null,      null,      null,      CAKE,       FALLTOTENNIS1, TENNIS3,   NOTSOSECRETKEY],
                                            [null,      null,      null,      null,       TENNIS0,       TENNIS1,   TENNIS2] ];
        public static const START_ROOM_ROW:uint = 6;
        public static const START_ROOM_COL:uint = 0;
    }
}