package ludumdare19.rooms
{
    import flash.display.BitmapData;
    import ludumdare19.gameobjects.TileMapWithUserData;
    import stego.util.MathUtil;
    import stego.util.ObjectPool;
	/**
     * ...
     * @author Michael Meyer
     */
    public class RoomLoader
    {
        /** tilemap we will need to clean up in unloadRoom */
        protected var tileMap:TileMapWithUserData;

        /** keep track of this mapping so that we can go from a TileMap to its TileSet object */
        protected var tileMapToTileSet:Array = [];

        public function unloadRoom():void
        {
            if (tileMap)
            {
                tileMap.clear();
                tileMap = null;
            }
            tileMapToTileSet = [];

            for each (var tileSet:TileSet in tileMapToTileSet)
            {
                tileSet.userdata = null;
                ObjectPool.returnObject(TileSet, tileSet);
            }
            tileMapToTileSet = [];
        }

        public function loadRoom(roomRow:uint, roomCol:uint, tileMap:TileMapWithUserData):void
        {
            unloadRoom();

            var roomData:RoomData = RoomData.ROOMS[roomRow][roomCol];

            var map:XML = roomData.xml;
            var cols:uint = uint(map.@width);
            var rows:uint = uint(map.@height);
            var tileWidth:uint = uint(map.@tilewidth);
            var tileHeight:uint = uint(map.@tileheight);

            // read map properties here
            for each (var property:XML in map.properties.property)
            {
                /*switch (property.@name.toString())
                {
                    case "name": roomData.name = property.@value; break;
                }*/
            }

            // get the starting GIDs from the tileset tags
            var tilesets:Object = { };
            for each (var tileset:XML in map.tileset)
            {
                tilesets[tileset.@name] = TileSet.fromXML(tileset);
            }

            // load the layers
            for each (var layer:XML in map.layer)
            {
                var layerName:String = layer.@name;
                if (layerName == "main")
                {
                    loadMainLayer(layer, roomData.tileSet, tilesets, tileWidth, tileHeight, tileMap);
                }
                else
                {
                    throw new Error("Don't know what to do with layer \""+layerName+"\"");
                }
            }
            // load objects
            for each (var object:XML in map.objectgroup.object)
            {
                var objectName:String = layer.@name;
                /*if ("player" == object.@name)
                {
                    addPlayer(object, tileWidth, tileHeight);
                }*/
            }

            // remember to clear it in unloadRoom
            this.tileMap = tileMap;
        }



        /**
         * Add tiles to the main layer
         * @param	layer           XML object for this layer
         * @param	tilesetImage    bitmapdata of image tiles
         * @param	tilesetInfo     object of TileSets we can look up by name
         * @param	mapTileWidth    width of each map tile
         * @param	mapTileHeight   height of each map tile
         * @param   tileMap         TileMap in which to load the room
         */
        private function loadMainLayer( layer:XML,
                                        tilesetImage:BitmapData,
                                        tilesetInfo:Object,
                                        mapTileWidth:Number,
                                        mapTileHeight:Number,
                                        tileMap:TileMapWithUserData ):void
        {
            var layerWidth:uint = layer.@width,
                layerHeight:uint = layer.@height,
                tilesetName:String = layer.properties.property.(@name == "tileset").@value,
                tileset:TileSet = tilesetInfo[tilesetName],
                firstgid:int = tileset.firstgid,
                tilesetWidth:uint = tileset.tilewidth,
                tilesetHeight:uint = tileset.tileheight,
                mapRowsPerTile:uint = Math.ceil(tilesetHeight / mapTileHeight),
                mapColsPerTile:uint = Math.ceil(tilesetWidth / mapTileWidth),
                mapRow:uint = 0,
                mapCol:uint = 0,
                index:int;

            tileMap.tilesetImage = tilesetImage;
            tileMap.tileWidth = MathUtil.roundUpToNearestMultiple(tilesetWidth, mapTileWidth);
            tileMap.tileHeight = MathUtil.roundUpToNearestMultiple(tilesetHeight, mapTileHeight);
            tileMap.width = layerWidth * mapTileWidth;
            tileMap.height = layerHeight * mapTileHeight

            for each (var tile:XML in layer.data.tile)
            {
                // when the map tiles don't match the tileset tiles, we have to be smart about skipping tiles
                if (((1 + mapRow) % mapRowsPerTile) == 0)
                {
                    if ((mapCol % mapColsPerTile) == 0 )
                    {
                        index = tile.@gid - firstgid;
                        if (index >= 0)
                        {
                            tileMap.setTileWithUserdata( mapCol / mapColsPerTile,
                                                         mapRow / mapRowsPerTile,
                                                         index,
                                                         tileset.userdata[index] );
                        }
                    }
                }

                ++mapCol;
                if (mapCol >= layerWidth)
                {
                    mapCol %= layerWidth;
                    ++mapRow;
                }
            }

            // extra bookkeeping
            tileMapToTileSet[tileMap] = tileset;
        }

        /**
         * look up the TileSet that we used with the given TileMap
         */
        public function getTileSet(tileMap:TileMapWithUserData):TileSet
        {
            return tileMapToTileSet[tileMap];
        }
    }
}