﻿package stego
{
    import flash.display.Stage;
    import flash.geom.Rectangle;
    import stego.GameState;
    import stego.debug.DebugWidget;
    import stego.util.InputUtil;
    import stego.util.CheatUtil;
    import stego.util.music.MusicUtil;
    import stego.util.PhysicsUtil;
    import stego.util.scene.Scene;
    import stego.util.TimeUtil;

    /**
     * common interface to game implementations
     * @author Mike Meyer
     */
    public interface GameInterface
    {
        /**
         * The Stage that the game is attached to.
         */
        function get stage () : Stage

        /**
         * see how much the game is zoomed in
         */
        function get zoom():Number

        /**
         * set how much the game is zoomed in
         */
        function set zoom(zoom:Number):void

        /**
         * "camera" position.  Where the _center_ of the screen is.
         */
        function get cameraX():Number
        function set cameraX(value:Number):void

        /**
         * "camera" position.  Where the _center_ of the screen is.
         */
        function get cameraY():Number
        function set cameraY(value:Number):void

        /**
         * Get and set the camera shake amount.
         * You want this to be zero most of the time!
         */
        function get cameraShake():Number
        function set cameraShake(value:Number):void

        /**
         * get and set how fast the camera can move.
         */
        function get cameraSpeed():Number
        function set cameraSpeed(value:Number):void
        /**
         * get and set the object that the camera tries to follow
         */
        function get cameraFollowTarget():GameObject
        function set cameraFollowTarget(value:GameObject):void

        /**
         * get and set the limits of camera scrolling.  can be null if you want no limits
         */
        function get cameraFollowRect():Rectangle
        function set cameraFollowRect(value:Rectangle):void

        /**
         * get and set the input utility for polling key/button states
         */
        function get inputUtil():InputUtil
        function set inputUtil(value:InputUtil):void

        /**
         * get and set the physics utility the game is using
         */
        function get physics():PhysicsUtil
        function set physics(value:PhysicsUtil):void

        /**
         * get how wide it thinks the stage is.
         * the actual stage size may change, but this will remain the same
         */
        function get stageWidth():Number

        /**
         * get how tall it thinks the stage is.
         * the actual stage size may change, but this will remain the same
         */
        function get stageHeight():Number

        /**
         * get the rect for what this thinks the stage is.
         * the actual stage size may change, but this will remain the same
         */
        function get stageRect():Rectangle

        /**
         * Get a reference to the music utility
         */
        function get music():MusicUtil

        /**
         * Get a reference to the cheat utility
         */
        function get cheats():CheatUtil

        /**
         * Get a reference to the SceneLayers utility
         */
        function get layers():Scene

        /**
         * Get a reference to the time utility
         */
        function get time():TimeUtil

        /**
         * access to debugging features
         */
        function get debug():DebugWidget

        /**
         * re-init.  Will get called automagically the first time.
         */
        function init(stageWidth:Number, stageHeight:Number):void

        /**
         * freeze time and show the pause screen
         */
        function pause():void

        /**
         * resume after pausing
         */
        function unpause():void

        /**
         * set a script to switch to once the current update call is finished
         */
        function changeGameState(nextState:GameStateInterface):void

        /**
         * clean up anything this coul dpossibly leave dangling
         */
        function shutdown():void
    }
}
