﻿package stego.debug
{
    import flash.display.BitmapData;
    import flash.events.MouseEvent;
    import flash.geom.Matrix;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import flash.utils.getQualifiedClassName;
    import stego.collections.LinkedList;
    import stego.constants.Colors;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.gameobject.CanvasGameObject;
    import stego.ui.TextButton;
    import stego.ui.TextField;
    import stego.util.collision.HitBox;
    import stego.util.graphics.Canvas;
    import stego.util.ObjectPool;
    import stego.util.scene.GameObjectHolder;
    import stego.util.StringUtil;

    /**
     * debugging tools
     * @author Mike Meyer
     */
    public class DebugWidget extends GameObject
    {
        protected var _log:TextField;
        protected var _poolStatus:TextField;
        protected var _reusablesStatus:TextField;
        protected var _profilingStatus:TextField;
        protected var _hitboxLayer:CanvasGameObject;

        protected static const BACKGROUND_ALPHA:Number = 0.6;

        /** pixels of extra space between the buttons */
        protected static const BUTTON_SPACING:Number = 20;

        /** pixels betwee the left side of the screen and where debug text starts */
        protected static const LEFT_MARGIN:Number = 10;

        /** avoid new'ing these if we can */
        protected var _tempRect:Rectangle = new Rectangle();

        /**
         *
         */
        override public function init(game:GameInterface):void
        {
            super.init(game);

            if (0 == numChildren) // hack to see if it has been init'ed
            {
                //createBackground(game.stageWidth, game.stageHeight);
                createTabBar();
                _log = createStatusTextField();
                _poolStatus = createStatusTextField();
                _reusablesStatus = createStatusTextField();
                _profilingStatus = createStatusTextField();
                _hitboxLayer = internalGameObjects.addAtIndex(1, CanvasGameObject) as CanvasGameObject;
                _hitboxLayer.width = game.stageWidth;
                _hitboxLayer.height = game.stageHeight;

                internalGameObjects.add(FpsCounter);
            }

            // default to log view
            goToLog();

            // starts out invisible
            this.visible = false;
        }

        /**
         * make the background to go behind everything
         */
        protected function createBackground( width:Number, height:Number ):void
        {
            graphics.beginFill(0x000000, BACKGROUND_ALPHA);
            graphics.drawRect(
                0,
                0,
                width,
                height );
            graphics.endFill();
        }

        /**
         * create the clickable list of tabs
         */
        protected function createTabBar():void
        {
            var button:GameObject;
            button = addButton(LEFT_MARGIN, "log", goToLog);
            button = addButton(button.x + button.width + BUTTON_SPACING, "pool", goToPoolStatus);
            button = addButton(button.x + button.width + BUTTON_SPACING, "reuse", goToReusablesStatus);
            button = addButton(button.x + button.width + BUTTON_SPACING, "profiling", goToProfiling);
            button = addButton(button.x + button.width + BUTTON_SPACING, "hitboxes", goToHitboxes);
        }

        /**
         * add a tab button
         */
        private function addButton(x:Number, label:String, callback:Function):GameObject
        {
            var tb:TextButton = internalGameObjects.add(TextButton) as TextButton;
            tb.setText(label);
            tb.x = x;
            tb.y = 20;
            tb.setFontColor(Colors.WHITE);
            tb.setHoverColor(Colors.YELLOW);
            tb.addEventListener(MouseEvent.CLICK, callback);
            return tb;
        }

        /**
         * react to press of the log button
         */
        private function goToLog(e:MouseEvent= null):void
        {
            hideAllTabs();
            _log.visible = true;
        }

        /**
         * react to press of the pool button
         */
        private function goToPoolStatus(e:MouseEvent= null):void
        {
            hideAllTabs();
            _poolStatus.visible = true;
        }

        /**
         * react to press of the reusables button
         */
        private function goToReusablesStatus(e:MouseEvent= null):void
        {
            hideAllTabs();
            _reusablesStatus.visible = true;
        }

        /**
         * react to press of the profiling button
         */
        private function goToProfiling(e:MouseEvent= null):void
        {
            hideAllTabs();
            _profilingStatus.visible = true;
        }

        /**
         *
         */
        private function goToHitboxes(e:MouseEvent = null):void
        {
            hideAllTabs();
            _hitboxLayer.visible = true;
        }

        /**
         * hide all the objects that are in these half-assed tabs
         */
        protected function hideAllTabs():void
        {
            _log.visible = false;
            _poolStatus.visible = false;
            _reusablesStatus.visible = false;
            _profilingStatus.visible = false;
            _hitboxLayer.visible = false;
        }

        /**
         * make the log display
         */
        protected function createLog():void
        {
            if (!_log)
            {
                _log = internalGameObjects.add(TextField) as TextField;
                _log.y = 40;
            }
        }

        /**
         * make a status display widget
         */
        protected function createStatusTextField():TextField
        {
            var tf:TextField = internalGameObjects.add(TextField) as TextField;
            tf.fontColor = Colors.WHITE;
            tf.x = LEFT_MARGIN;
            tf.y = 60;
            return tf;
        }

        /**
         * clear the log
         */
        public function clearLog():void
        {
            if (_log)
            {
                _log.setText("");
            }
        }

        /**
         * add to the log
         */
        public function log(text:String):void
        {
            if (_log)
            {
                // TODO:  make into a ginormous scrolling thing someday
                _log.setText(_log.getText()+"\n"+text);
            }
        }

        /**
         *
         */
        override public function update(elapsedTime:Number):void
        {
            super.update(elapsedTime);
            var text:String, c:Object, key:String, ll:LinkedList;

            if (visible)
            {
                if (_poolStatus && _poolStatus.visible)
                {
                    text = "object pool info\n'new's:  " + (ObjectPool.numNews + ObjectPool.numBitmapDataNews + ObjectPool.numLinkedListNodeNews );
                    for (c in ObjectPool.pools)
                    {
                        ll = ObjectPool.pools[c];
                        text += "\n" + getQualifiedClassName(c) + ":  " + ll.count;
                    }
                    for (key in ObjectPool.bitmapDataPool)
                    {
                        ll = ObjectPool.bitmapDataPool[key];
                        text += "\nbitmapData " + key + ":  " + ll.count;
                    }
                    text += "\nlinkedListNode:  " + ObjectPool.linkedListNodePool.length;

                    _poolStatus.setText(text);
                }
                if (_reusablesStatus && _reusablesStatus.visible)
                {
                    text = "reusable game objects info\n'new's:  " + GameObjectHolder.numNews;
                    for (c in GameObjectHolder.reusables)
                    {
                        var a:Array = GameObjectHolder.reusables[c];
                        text += "\n" + getQualifiedClassName(c) + ":  " + a.length;
                    }
                    _reusablesStatus.setText(text);
                }
                if (_profilingStatus && _profilingStatus.visible)
                {
                    text = "Profiling info\n";
                    text += StringUtil.fixedWidth("name", 25);
                    text += StringUtil.fixedWidth("times", 8);
                    text += StringUtil.fixedWidth("avg", 6);
                    text += StringUtil.fixedWidth("min", 4);
                    text += StringUtil.fixedWidth("max", 4);
                    text += StringUtil.fixedWidth("total", 6);
                    for (var sectionName:String in Profiler.sections)
                    {
                        var ps:ProfilerSection = Profiler.sections[sectionName];
                        text += "\n" + StringUtil.fixedWidth(sectionName, 25);
                        text += StringUtil.fixedWidth(ps.times.toString(), 8);
                        text += StringUtil.fixedWidth(ps.avg.toFixed(3), 6);
                        text += StringUtil.fixedWidth(ps.min.toString(), 4);
                        text += StringUtil.fixedWidth(ps.max.toString(), 4);
                        text += StringUtil.fixedWidth(ps.total.toString(), 6);
                    }
                    _profilingStatus.setText(text);
                }
            }
        }

        /**
         * cleanup
         */
        override public function shutdown():void
        {
            super.shutdown();
            game = null;
        }

        /**
         * draw the hitboxes for the objects in the layer
         */
        protected function drawHitboxesInLayer(layer:GameObjectHolder):void
        {
            layer.forEach(drawHitboxesForObject);
        }

        /**
         * draw the hitboxes for the object
         */
        protected function drawHitboxesForObject(go:GameObject):void
        {
            var canvas:Canvas = _hitboxLayer.canvas,
                hitboxes:Array = go.getHitboxes(),
                matrix:Matrix = go.transform.concatenatedMatrix,
                point1:Point,
                point2:Point;
            for each (var hitbox:HitBox in hitboxes)
            {
                point1 = matrix.transformPoint(hitbox.topLeft);
                point2 = matrix.transformPoint(hitbox.bottomRight);

                _tempRect.x = Math.min(point1.x, point2.x);
                _tempRect.y = Math.min(point1.y, point2.y);
                _tempRect.width = Math.abs(point1.x - point2.x);
                _tempRect.height = Math.abs(point1.y - point2.y);

                canvas.fillRectBlended(_tempRect, (0xFFFFFF & hitbox.color) | 0x7F000000);
                //canvas.fillRect(_tempRect, 0xFF000000 | hitbox.color);
            }
            ObjectPool.returnArray(hitboxes, HitBox);
        }
        override public function preDraw(timeDelta:Number):void
        {
            super.preDraw(timeDelta);
            if (visible)
            {
                if (_hitboxLayer.visible)
                {
                    var canvas:Canvas = _hitboxLayer.canvas;
                    canvas.clear();
                    game.layers.layers.forEach(drawHitboxesInLayer);
                }
            }
        }
    }
}
