package stego.gameobject
{
    import flash.display.Bitmap;
    import flash.display.PixelSnapping;
    import flash.geom.Rectangle;
    import stego.GameInterface;
    import stego.GameObject;
    import stego.util.graphics.Canvas;

    /**
     * A game object with a built-in stego.util.graphics.Canvas
     * @author Mike Meyer
     */
    public class CanvasGameObject extends GameObject
    {
        /** width of the entire tileset image */
        override public function get width():Number { return _canvas.width; }
        override public function set width(value:Number):void
        {
            _canvas.width = value;
        }

        /** height of the entire tileset image */
        override public function get height():Number { return _canvas.height; }
        override public function set height(value:Number):void
        {
            _canvas.height = value;
        }

        /** canvas we draw onto */
        public function get canvas():Canvas { return _canvas; }
        protected var _canvas:Canvas;

        /** the bitmap used to render it */
        protected var _bitmap:Bitmap;

        /** avoid new'ing these if we can */
        protected var tempRect:Rectangle = new Rectangle();

        /**
         * initial setup and preparation for reuse
         */
        override public function init(game:GameInterface):void
        {
            super.init(game);

            if (!_canvas)
            {
                _bitmap = new Bitmap(null, PixelSnapping.NEVER);
                addChild(_bitmap);
                _canvas = new Canvas(_bitmap);
            }
            else
            {
                _canvas.clear();
            }
        }

        /**
         * fades out the canvas contents
         * color values should be between zero and one
         * @param   amount  how much to fade the canvas by. [0-1] 0 is not at all. 1 makes it completely transparent
         */
        public function fadeTowardColor(red:Number, green:Number, blue:Number, alpha:Number, amount:Number):void
        {
            _canvas.fadeTowardColor(red, green, blue, alpha, amount);
        }
    }
}
