﻿package stego.ui
{
    import flash.events.Event;
    import flash.text.TextFieldAutoSize;
    import flash.text.TextFieldType;
    import flash.text.TextFormat;
    import stego.GameInterface;
    import stego.GameObject;

    /**
     * wrapper around flash textfields for acting something like a standard windows textfield
     * Supports the same Event.CHANGE event that flash textfields do.
     * @author Mike Meyer
     */
    public class InputText extends GameObject
    {
        /**
         * actual textfield to wrap
         */
        protected var _textField:flash.text.TextField;

        private static const DEFAULT_WORDWRAP_WIDTH:Number = 200;

        public function get wordWrap():Boolean { return _textField.wordWrap; }
        public function set wordWrap(value:Boolean ):void { _textField.wordWrap = value; }

        public function get wordWrapWidth():Number { return _textField.width; }
        public function set wordWrapWidth(value:Number):void { _textField.width = value; }

        /**
         * get/set the displayed text.
         */
        public function get text():String { return _textField.text; }
        public function set text(desiredText:String):void { _textField.text = desiredText; }

        /**
         * get/set the displayed text.
         */
        public function get htmlText():String { return _textField.htmlText; }
        public function set htmlText(desiredHtmlText:String):void{ _textField.htmlText = desiredHtmlText; }

        /**
         * get/set whether the textfield allows newlines
         */
        public function get multiline():Boolean { return _textField.multiline; }
        public function set multiline(value:Boolean):void{ _textField.multiline = value; }


        /**
         * get/set the font.
         */
        public function get font():String
        {
            var fmt:TextFormat = _textField.getTextFormat();
            return fmt.font;
        }
        public function set font(fontName:String):void
        {
            var fmt:TextFormat = _textField.getTextFormat();
            fmt.font = fontName;
            _textField.defaultTextFormat = fmt;
            _textField.setTextFormat(fmt);
        }

        /**
         * get/set the font size.
         */
        public function get fontSize():Number
        {
            var fmt:TextFormat = _textField.getTextFormat();
            return Number(fmt.size);
        }
        public function set fontSize(desiredSize:Number):void
        {
            var fmt:TextFormat = _textField.getTextFormat();
            fmt.size = desiredSize;
            _textField.defaultTextFormat = fmt;
            _textField.setTextFormat(fmt);
        }

        /**
         * get/set the color of the displayed text
         */
        public function get fontColor():Number { return _textField.textColor; }
        public function set fontColor(color:Number):void { _textField.textColor = color; }


        public function InputText():void
        {
            super();

            if (null == _textField)
            {
                _textField = new flash.text.TextField();
                _textField.autoSize = TextFieldAutoSize.LEFT;
                _textField.multiline = false;
                _textField.type = TextFieldType.INPUT;
                _textField.addEventListener(Event.CHANGE, textInputHandler);
                this.addChild(_textField);
            }
        }

        /**
         * Just relay the event along
         */
        private function textInputHandler(e:Event):void
        {
            dispatchEvent(e);
        }

        /**
         * this function called for both initial setup and to re-init for reuse
         */
        public override function init(game:GameInterface):void
        {
            super.init(game);
            resetDefaults();
        }

        /**
         * reset to defaults.
         */
        private function resetDefaults():void
        {
            _textField.x = 0;
            _textField.y = 0;
            _textField.selectable = true;
            _textField.embedFonts = true;

            var format:TextFormat = new TextFormat();
            format.font = TextField.defaultFont;
            format.size = TextField.defaultFontSize;
            _textField.defaultTextFormat = format;
            _textField.setTextFormat(format);
            _textField.textColor = TextField.defaultFontColor;
            _textField.text = "";

            wordWrap = false;
        }

        /**
         * Give Flash's focus to the textfield
         */
        public function grabFocus():void
        {
            stage.focus = _textField;
        }

        /**
         * Select everything so that when they start typing it gets replaced
         */
        public function selectAll():void
        {
            _textField.setSelection(0, _textField.text.length);
        }
    }
}
