﻿package stego.ui
{
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import stego.constants.Colors;
    import stego.GameInterface;
    import stego.GameObject;

    /**
     * Clickable text with rollover color change
     * @author Mike Meyer
     */
    public class TextButton extends GameObject
    {
        /** default color to use for the rollover effects */
        public static var defaultHighlightColor:uint = Colors.WHITE;

        private var _hoverColor:Number = defaultHighlightColor;
        private var _mouseEventDummy:Sprite;
        private var _textField:TextField;
        protected var color:Number = TextField.defaultFontColor;

        public override function init(game:GameInterface):void
        {
            super.init(game);

            if (null == _textField)
            {
                _textField = new TextField();
                _textField.fontColor = color;
                this.addChild(_textField);
            }
            else
            {
                _textField.resetDefaults();
            }

            if (null == _mouseEventDummy)
            {
                _mouseEventDummy = new Sprite();
                _mouseEventDummy.graphics.beginFill(0, 0);
                _mouseEventDummy.graphics.drawRect(0, 0, 1, 1);
                _mouseEventDummy.graphics.endFill();
                this.addChild(_mouseEventDummy);
            }

            matchDummyToText();

            addMouseEventListeners();
        }

        public function setHoverColor(color:Number):void
        {
            _hoverColor = color;
        }

        public override function shutdown():void
        {
            removeMouseEventListeners();
            super.shutdown();
        }

        private function addMouseEventListeners():void
        {
            _mouseEventDummy.addEventListener(MouseEvent.ROLL_OVER, handleRollOver);
            _mouseEventDummy.addEventListener(MouseEvent.ROLL_OUT, handleRollOut);

            this.addEventListener(MouseEvent.CLICK, fixFocus);
        }

        private function removeMouseEventListeners():void
        {
            _mouseEventDummy.removeEventListener(MouseEvent.ROLL_OVER, handleRollOver);
            _mouseEventDummy.removeEventListener(MouseEvent.ROLL_OUT, handleRollOut);

            this.removeEventListener(MouseEvent.CLICK, fixFocus);
        }

        private function handleRollOver(evt:MouseEvent = null):void
        {
            _textField.fontColor = _hoverColor;
        }

        private function handleRollOut(evt:MouseEvent = null):void
        {
            _textField.fontColor = color;
        }

        /**
         * workaround for weird flash issue with inputs after clicking a textfield
         */
        private function fixFocus(e:Event = null):void
        {
            // working around flash issue with clicked textfield making the keyboard not work.
            stage.focus = null;
        }

        /**
         * make the dummy mouse event rect match the displayed text
         */
        private function matchDummyToText():void
        {
            var bounds:Rectangle = _textField.getBounds(_mouseEventDummy.parent);
            _mouseEventDummy.x = bounds.x;
            _mouseEventDummy.y = bounds.y;
            _mouseEventDummy.width = bounds.width;
            _mouseEventDummy.height = bounds.height;
        }

        public function centerOn(x:Number, y:Number):void
        {
            this.x = x;
            this.y = y;
            _textField.hAlign = TextField.HALIGN_CENTER;
            _textField.vAlign = TextField.VALIGN_MIDDLE;
            matchDummyToText();
        }
        public function setFont(font:String):void
        {
            _textField.setFont(font);
            matchDummyToText();
        }
        public function setFontSize(desiredSize:Number):void
        {
            _textField.setFontSize(desiredSize);
            matchDummyToText();
        }
        public function setFontColor(color:Number):void
        {
            this.color = color;
            _textField.fontColor = color;
        }
        public function setText(text:String):void
        {
            _textField.setText(text);
            matchDummyToText();
        }
        public function getText():String
        {
            return _textField.getText();
        }
    }

}
