﻿package stego.util
{
    /**
     * static functions to help with dealing with this game's faked perspective/3d
     * this is all based on
     * magic perspective formula. © 1498 Leonardo da Vinvi
     *   apparent_size = ( 1 / ( 1 + ( (z - REALSIZE_Z) * _zscale) ) );
     *   _zscale = 2.0 * Math.tan(vertical_viewing_angle) / canvasHeight
     * @author Mike Meyer
     */
    public class PerspectiveUtil
    {
        /// z value at which object's appear their "real" size
        private static const REALSIZE_Z:Number = 0;

        /**
         * find out the scaling value based on an object's depth.
         * @param	z - distance back from the "real size" depth
         * @param	fovY - vertical field of view in radians
         * @param	canvasHeight - height of the area we are drawing to
         * @return
         */
        public static function getScaleFromZ(z:Number, fovY:Number, canvasHeight:Number):Number
        {
            var zscale:Number = 2.0 * Math.tan(fovY) / canvasHeight;
            return ( 1 / ( 1 + ( (z - REALSIZE_Z) * zscale) ) );
        }

        /**
         * inverse operation of getScaleFromZ. basically just for setting my z step based on the art
         * @param	z - distance back from the "real size" depth
         * @param	fovY - vertical field of view in radians
         * @param	canvasHeight - height of the area we are drawing to
         * @return
         */
        public static function getZFromScale(scale:Number, fovY:Number, canvasHeight:Number):Number
        {
            var zscale:Number = 2.0 * Math.tan(fovY) / canvasHeight;
            return ((((1.0 / scale) - 1) / zscale) + REALSIZE_Z);
        }
    }

}
